import _extends from "@babel/runtime/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import { initDomain } from '@resolve-js/core';
import { createQuery } from '@resolve-js/runtime';
import { getSecretsManager } from '../../runtime/get-secrets-manager';
import { getEventStore } from '../../runtime/get-event-store';
import { getSagaRuntime } from '../../runtime/get-saga-runtime';
import { mockSideEffects } from '../../runtime/mock-side-effects';
import { getReadModelAdapter } from '../../runtime/get-read-model-adapter';
import partial from 'lodash.partial';
import { defaultAssertion } from '../../utils/assertions';
import { getCommandImplementationKey, getQueryImplementationKey } from '../../runtime/utils';

var promisedAssertion = function promisedAssertion(assertion, result, error) {
  return new Promise(function (resolve, reject) {
    return assertion(resolve, reject, result, error, false);
  });
};

export var makeTestEnvironment = function makeTestEnvironment(context) {
  var mockedCommandImplementations = new Map();
  var mockedQueryImplementations = new Map();
  var executed = false;
  var useRealSideEffects = false;
  var sideEffectsStartTimestamp = 0;
  var assertions = [];
  var secretsManager = getSecretsManager();
  var completeTest;
  var failTest;

  var setSecretsManager = function setSecretsManager(value) {
    secretsManager = value;
  };

  var allowSideEffects = function allowSideEffects() {
    useRealSideEffects = true;
  };

  var setSideEffectsStartTimestamp = function setSideEffectsStartTimestamp(value) {
    sideEffectsStartTimestamp = value;
  };

  var addAssertion = function addAssertion(value) {
    assertions.push(partial(promisedAssertion, value));
  };

  var isExecuted = function isExecuted() {
    return executed;
  };

  var mockCommandImplementation = function mockCommandImplementation(aggregateName, type, implementation) {
    mockedCommandImplementations.set(getCommandImplementationKey({
      type: type,
      aggregateName: aggregateName
    }), implementation);
  };

  var mockQueryImplementation = function mockQueryImplementation(modelName, resolverName, implementation) {
    mockedQueryImplementations.set(getQueryImplementationKey({
      modelName: modelName,
      resolverName: resolverName
    }), implementation);
  };

  var promise = new Promise(function (resolve, reject) {
    completeTest = resolve;
    failTest = reject;
  });

  var execute = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4() {
      var result, saga, events, adapter, encryption, actualEncryption, actualAdapter, domain, liveErrors, monitoring, eventstoreAdapter, errors, executor, runtime, status, liveErrorIndex, _error2, summaryError;

      return _regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              executed = true;
              result = Object.defineProperty({
                commands: [],
                scheduledCommands: [],
                queries: [],
                sideEffects: []
              }, // FIXME: deprecated
              'scheduleCommands', {
                enumerable: false,
                get: function get() {
                  // eslint-disable-next-line no-console
                  console.warn("'scheduleCommands' property deprecated, use 'scheduledCommands' instead.");
                  return this.scheduledCommands;
                }
              });
              saga = context.saga, events = context.events, adapter = context.adapter, encryption = context.encryption;
              actualEncryption = encryption != null ? encryption : /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
                return _regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt("return", null);

                      case 1:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              if (!(adapter != null)) {
                _context4.next = 8;
                break;
              }

              _context4.t0 = adapter;
              _context4.next = 11;
              break;

            case 8:
              _context4.next = 10;
              return getReadModelAdapter();

            case 10:
              _context4.t0 = _context4.sent;

            case 11:
              actualAdapter = _context4.t0;
              domain = initDomain({
                viewModels: [],
                readModels: [],
                aggregates: [],
                sagas: [{
                  name: saga.name,
                  handlers: saga.handlers,
                  sideEffects: useRealSideEffects ? saga.sideEffects : mockSideEffects(result, saga.sideEffects),
                  connectorName: 'ADAPTER_NAME',
                  encryption: actualEncryption
                }]
              });
              liveErrors = [];
              monitoring = {
                group: function group(config) {
                  if (config.Part !== 'ReadModelProjection') {
                    return monitoring;
                  }

                  return _extends({}, monitoring, {
                    error: function (_error) {
                      function error(_x) {
                        return _error.apply(this, arguments);
                      }

                      error.toString = function () {
                        return _error.toString();
                      };

                      return error;
                    }(function (error) {
                      liveErrors.push(error);
                    })
                  });
                },
                time: function time() {
                  return void 0;
                },
                timeEnd: function timeEnd() {
                  return void 0;
                },
                error: function error() {
                  return void 0;
                },
                publish: function () {
                  var _publish = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
                    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            return _context2.abrupt("return", void 0);

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  function publish() {
                    return _publish.apply(this, arguments);
                  }

                  return publish;
                }()
              };
              eventstoreAdapter = getEventStore(events);
              errors = [];
              executor = null;
              _context4.prev = 18;
              runtime = getSagaRuntime(result, {
                commands: mockedCommandImplementations,
                queries: mockedQueryImplementations
              }, domain.sagaDomain.schedulerName, secretsManager, monitoring, sideEffectsStartTimestamp);
              executor = createQuery({
                applicationName: 'APP_NAME',
                readModelConnectors: {
                  ADAPTER_NAME: actualAdapter
                },
                getVacantTimeInMillis: function getVacantTimeInMillis() {
                  return 0x7fffffff;
                },
                invokeEventSubscriberAsync: function () {
                  var _invokeEventSubscriberAsync = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
                    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            return _context3.abrupt("return", void 0);

                          case 1:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3);
                  }));

                  function invokeEventSubscriberAsync() {
                    return _invokeEventSubscriberAsync.apply(this, arguments);
                  }

                  return invokeEventSubscriberAsync;
                }(),
                eventstoreAdapter: eventstoreAdapter,
                readModelsInterop: domain.sagaDomain.acquireSagasInterop(runtime),
                viewModelsInterop: {},
                performanceTracer: null
              });
              _context4.prev = 21;
              _context4.next = 24;
              return eventstoreAdapter.ensureEventSubscriber({
                applicationName: 'APP_NAME',
                eventSubscriber: saga.name,
                status: null,
                destination: 'LOCAL'
              });

            case 24:
              _context4.next = 26;
              return executor.subscribe({
                modelName: saga.name,
                subscriptionOptions: {
                  eventTypes: null,
                  aggregateIds: null
                }
              });

            case 26:
              _context4.next = 28;
              return executor.resume({
                modelName: saga.name
              });

            case 28:
              _context4.next = 30;
              return executor.build({
                modelName: saga.name
              });

            case 30:
              _context4.next = 32;
              return executor.build({
                modelName: saga.name
              });

            case 32:
              _context4.next = 34;
              return executor.status({
                modelName: saga.name
              });

            case 34:
              status = _context4.sent;

              if (!(status.status === 'error')) {
                _context4.next = 46;
                break;
              }

              if (Array.isArray(status.errors)) {
                _context4.next = 38;
                break;
              }

              throw new Error('Unknown error');

            case 38:
              liveErrorIndex = status.errors.length === 1 ? liveErrors.findIndex(function (err) {
                return err.message === status.errors[0].message;
              }) : -1;

              if (!(liveErrorIndex < 0)) {
                _context4.next = 45;
                break;
              }

              _error2 = new Error(status.errors.map(function (err) {
                return err.message;
              }).join('\n'));
              _error2.stack = status.errors.map(function (err) {
                return err.stack;
              }).join('\n');
              throw _error2;

            case 45:
              throw liveErrors[liveErrorIndex];

            case 46:
              _context4.next = 51;
              break;

            case 48:
              _context4.prev = 48;
              _context4.t1 = _context4["catch"](21);
              errors.push(_context4.t1);

            case 51:
              _context4.prev = 51;
              _context4.next = 54;
              return executor.unsubscribe({
                modelName: saga.name
              });

            case 54:
              _context4.next = 59;
              break;

            case 56:
              _context4.prev = 56;
              _context4.t2 = _context4["catch"](51);
              errors.push(_context4.t2);

            case 59:
              _context4.prev = 59;
              _context4.next = 62;
              return eventstoreAdapter.removeEventSubscriber({
                applicationName: 'APP_NAME',
                eventSubscriber: saga.name
              });

            case 62:
              _context4.next = 67;
              break;

            case 64:
              _context4.prev = 64;
              _context4.t3 = _context4["catch"](59);
              errors.push(_context4.t3);

            case 67:
              _context4.next = 72;
              break;

            case 69:
              _context4.prev = 69;
              _context4.t4 = _context4["catch"](18);
              errors.push(_context4.t4);

            case 72:
              _context4.prev = 72;

              if (!(executor != null)) {
                _context4.next = 82;
                break;
              }

              _context4.prev = 74;
              _context4.next = 77;
              return executor.dispose();

            case 77:
              _context4.next = 82;
              break;

            case 79:
              _context4.prev = 79;
              _context4.t5 = _context4["catch"](74);
              errors.push(_context4.t5);

            case 82:
              return _context4.finish(72);

            case 83:
              if (!(errors.length === 0)) {
                _context4.next = 99;
                break;
              }

              if (!(assertions.length > 0)) {
                _context4.next = 96;
                break;
              }

              _context4.prev = 85;
              _context4.next = 88;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, null);
              }));

            case 88:
              completeTest(result);
              _context4.next = 94;
              break;

            case 91:
              _context4.prev = 91;
              _context4.t6 = _context4["catch"](85);
              failTest(_context4.t6);

            case 94:
              _context4.next = 97;
              break;

            case 96:
              defaultAssertion(completeTest, failTest, result, null);

            case 97:
              _context4.next = 115;
              break;

            case 99:
              summaryError = errors[0];

              if (errors.length > 1) {
                summaryError = new Error(errors.map(function (err) {
                  return err.message;
                }).join('\n'));
                summaryError.stack = errors.map(function (err) {
                  return err.stack;
                }).join('\n');
              } // eslint-disable-next-line no-console


              console.error(summaryError);

              if (!(assertions.length > 0)) {
                _context4.next = 114;
                break;
              }

              _context4.prev = 103;
              _context4.next = 106;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, errors[0]);
              }));

            case 106:
              failTest(errors[0]);
              _context4.next = 112;
              break;

            case 109:
              _context4.prev = 109;
              _context4.t7 = _context4["catch"](103);
              failTest(_context4.t7);

            case 112:
              _context4.next = 115;
              break;

            case 114:
              defaultAssertion(completeTest, failTest, result, errors[0]);

            case 115:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, null, [[18, 69, 72, 83], [21, 48], [51, 56], [59, 64], [74, 79], [85, 91], [103, 109]]);
    }));

    return function execute() {
      return _ref.apply(this, arguments);
    };
  }();

  setImmediate(execute);
  return {
    setSecretsManager: setSecretsManager,
    allowSideEffects: allowSideEffects,
    setSideEffectsStartTimestamp: setSideEffectsStartTimestamp,
    addAssertion: addAssertion,
    isExecuted: isExecuted,
    promise: promise,
    mockCommandImplementation: mockCommandImplementation,
    mockQueryImplementation: mockQueryImplementation
  };
};
//# sourceMappingURL=make-test-environment.js.map