"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.makeTestEnvironment = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _core = require("@resolve-js/core");

var _runtime = require("@resolve-js/runtime");

var _assertions = require("../../utils/assertions");

var _getSecretsManager = require("../../runtime/get-secrets-manager");

var _getEventStore = require("../../runtime/get-event-store");

var makeTestEnvironment = function makeTestEnvironment(context) {
  var executed = false;
  var authToken;
  var assertion;
  var secretsManager = (0, _getSecretsManager.getSecretsManager)();
  var completeTest;
  var failTest;

  var setAuthToken = function setAuthToken(value) {
    authToken = value;
  };

  var setSecretsManager = function setSecretsManager(value) {
    secretsManager = value;
  };

  var setAssertion = function setAssertion(value) {
    assertion = value;
  };

  var getAssertion = function getAssertion() {
    return assertion;
  };

  var isExecuted = function isExecuted() {
    return executed;
  };

  var promise = new Promise(function (resolve, reject) {
    completeTest = resolve;
    failTest = reject;
  });

  var execute = /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
      var aggregate, events, command, _context$aggregateId, aggregateId, domain, executor, actualAssertion, commandResult, testResult;

      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              executed = true;
              aggregate = context.aggregate, events = context.events, command = context.command, _context$aggregateId = context.aggregateId, aggregateId = _context$aggregateId === void 0 ? 'test-aggregate-id' : _context$aggregateId;
              domain = (0, _core.initDomain)({
                viewModels: [],
                readModels: [],
                aggregates: [{
                  name: aggregate.name,
                  projection: aggregate.projection,
                  commands: aggregate.commands,
                  encryption: aggregate.encryption || null,
                  deserializeState: JSON.parse,
                  serializeState: JSON.stringify,
                  invariantHash: 'invariant-hash'
                }],
                sagas: []
              });
              executor = null;
              actualAssertion = assertion != null ? assertion : _assertions.defaultAssertion;
              _context.prev = 5;
              executor = (0, _runtime.createCommand)({
                performanceTracer: null,
                aggregatesInterop: domain.aggregateDomain.acquireAggregatesInterop({
                  eventstore: (0, _getEventStore.getEventStore)(events, {
                    aggregateId: aggregateId
                  }),
                  secretsManager: secretsManager,
                  hooks: {}
                })
              });
              _context.next = 9;
              return executor({
                aggregateId: aggregateId,
                aggregateName: aggregate.name,
                type: command.name,
                payload: command.payload || {},
                jwt: authToken
              });

            case 9:
              commandResult = _context.sent;
              testResult = {
                type: commandResult.type
              };

              if (Object.prototype.hasOwnProperty.call(commandResult, 'payload')) {
                testResult.payload = commandResult.payload;
              }

              return _context.abrupt("return", actualAssertion(completeTest, failTest, testResult, null, false));

            case 15:
              _context.prev = 15;
              _context.t0 = _context["catch"](5);
              return _context.abrupt("return", actualAssertion(completeTest, failTest, null, _context.t0, false));

            case 18:
              _context.prev = 18;

              if (!executor) {
                _context.next = 22;
                break;
              }

              _context.next = 22;
              return executor.dispose();

            case 22:
              return _context.finish(18);

            case 23:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[5, 15, 18, 23]]);
    }));

    return function execute() {
      return _ref.apply(this, arguments);
    };
  }();

  setImmediate(execute);
  return {
    setAuthToken: setAuthToken,
    setSecretsManager: setSecretsManager,
    setAssertion: setAssertion,
    getAssertion: getAssertion,
    isExecuted: isExecuted,
    promise: promise
  };
};

exports.makeTestEnvironment = makeTestEnvironment;
//# sourceMappingURL=make-test-environment.js.map